@@ -94,6 +94,8 @@ class SpeedPosition(FormationAlgorithm):
             LOG.warning("The time limit for the solver should be at least 2s! Otherwise it may not be possible for the solver to produce a solution (especially with many vehicles)!")
         self._formation_centralized_kind = formation_centralized_kind  # the kind of the centralized formation
         self._solver_time_limit = solver_time_limit  # the time limit for the optimal solver per assignment problem
+        if formation_centralized_kind == "optimal":
+            LOG.warning("NOTE: The optimal solver is still experimental!")
 
         # statistics
         self._assignments_solved = 0
@@ -208,11 +210,13 @@ class SpeedPosition(FormationAlgorithm):
         if isinstance(self._owner, Infrastructure):
             LOG.info(f"{self._owner.iid} is running formation algorithm {self._name} ({self._formation_centralized_kind}) at {self._owner._simulator.step}")
             if self._formation_centralized_kind == 'optimal':
-                self._do_formation_optimal()
+                # optimal
+                self._do_formation_optimal()  # still experimental
             else:
                 # greedy
                 self._do_formation_centralized()
         else:
+            # greedy
             self._do_formation_distributed()
 
     def finish(self):
@@ -225,11 +229,11 @@ class SpeedPosition(FormationAlgorithm):
         # write statistics
         if not self._owner._simulator._record_platoon_formation:
             return
-        if self._owner._formation_kind != 'optimal':
+        if self._formation_centralized_kind != 'optimal':
             return
 
         from ..infrastructure import Infrastructure
-        assert(isinstance(self._owner, Infrastructure))
+        assert isinstance(self._owner, Infrastructure)
 
         if self._owner._simulator._record_infrastructure_assignments:
             record_infrastructure_assignments(
@@ -454,6 +458,8 @@ class SpeedPosition(FormationAlgorithm):
         Run centralized optimal formation approach.
 
         This selects candidates and triggers join maneuvers.
+
+        NOTE: This functionality is still experimental!
         """
 
         from ortools.linear_solver import pywraplp
@@ -652,22 +658,22 @@ class SpeedPosition(FormationAlgorithm):
                     continue
                 if target_platoon.platoon_id != mapping['pid']:
                     # meanwhile, the leader became a platoon member
-                    assert(leader.is_in_platoon() and leader.platoon_role == PlatoonRole.FOLLOWER)
+                    assert (leader.is_in_platoon() and leader.platoon_role == PlatoonRole.FOLLOWER)
                     LOG.warning(f"{vehicle.vid}'s assigned platoon {mapping['pid']} (leader {leader.vid}) meanwhile joined another platoon {target_platoon.platoon_id}!")
                     self._assignments_candidate_joined_already += 1
                     continue
                 else:
-                    assert(not leader.in_maneuver)
-                    assert(not leader.is_in_platoon() or leader.platoon_role == PlatoonRole.LEADER)
+                    assert not leader.in_maneuver
+                    assert (not leader.is_in_platoon() or leader.platoon_role == PlatoonRole.LEADER)
                 # let vehicle join platoon
                 if vehicle.is_in_platoon():
                     # meanwhile, we became a platoon leader
-                    assert(vehicle.platoon_role == PlatoonRole.LEADER)
+                    assert vehicle.platoon_role == PlatoonRole.LEADER
                     LOG.warning(f"{vehicle.vid} meanwhile became the leader of platoon {vehicle.platoon.platoon_id}. Hence, no assignment is possible/necessary anymore")
                     self._assingments_vehicle_became_leader += 1
                     continue
-                assert(not vehicle.in_maneuver)
-                assert(not leader.in_maneuver)
+                assert not vehicle.in_maneuver
+                assert not leader.in_maneuver
 
                 # actual join
                 vehicle._join(target_platoon.platoon_id, target_platoon.leader.vid)
