@@ -20,6 +20,7 @@ import argparse
 import logging
 import os
 import sys
+import textwrap
 import time
 
 import pandas
@@ -28,6 +29,7 @@ from tqdm import tqdm
 from plafosim import __version__
 from plafosim.gui import (
     add_gui_vehicle,
+    change_gui_vehicle_color,
     close_gui,
     gui_step,
     move_gui_vehicle,
@@ -35,7 +37,7 @@ from plafosim.gui import (
     start_gui,
 )
 from plafosim.simulator import DEFAULTS
-from plafosim.util import find_resource
+from plafosim.util import find_resource, hex2rgb
 
 LOG = logging.getLogger(__name__)
 
@@ -45,32 +47,36 @@ tools = os.path.join(os.environ['SUMO_HOME'], 'tools')
 sys.path.append(tools)
 
 
-class CustomFormatter(argparse.ArgumentDefaultsHelpFormatter,
-                      argparse.RawDescriptionHelpFormatter,
-                      argparse.MetavarTypeHelpFormatter):
+class CustomFormatter(
+    argparse.ArgumentDefaultsHelpFormatter,
+    argparse.RawDescriptionHelpFormatter,
+    argparse.MetavarTypeHelpFormatter,
+):
     """Metaclass combining multiple formatter classes for argparse"""
+
     pass
 
 
+# TODO duplicated code with main script
 def parse_args() -> argparse.Namespace:
     # parse some parameters
     parser = argparse.ArgumentParser(
         formatter_class=CustomFormatter,
         allow_abbrev=False,
-        description="""
-    Platoon Formation Simulator (PlaFoSim) -- A simple and scalable simulator for platoon formation.
+        description=textwrap.dedent("""
+            Platoon Formation Simulator (PlaFoSim) -- A simple and scalable simulator for platoon formation.
 
-    Copyright (c) 2020-2021 Julian Heinovski <heinovski@ccs-labs.org>
-    This program comes with ABSOLUTELY NO WARRANTY.
-    This is free software, and you are welcome to redistribute it under certain conditions.
+            Copyright (c) 2020-2021 Julian Heinovski <heinovski@ccs-labs.org>
+            This program comes with ABSOLUTELY NO WARRANTY.
+            This is free software, and you are welcome to redistribute it under certain conditions.
 
-    If you are working with PlaFoSim, please cite the following paper:
+            If you are working with PlaFoSim, please cite the following paper:
 
-    Julian Heinovski, Dominik S. Buse and Falko Dressler,
-    "Scalable Simulation of Platoon Formation Maneuvers with PlaFoSim,"
-    Proceedings of 13th IEEE Vehicular Networking Conference (VNC 2021),
-    Poster Session, Virtual Conference, November 2021.
-    """,
+            Julian Heinovski, Dominik S. Buse and Falko Dressler,
+            "Scalable Simulation of Platoon Formation Maneuvers with PlaFoSim,"
+            Proceedings of 13th IEEE Vehicular Networking Conference (VNC 2021),
+            Poster Session, Virtual Conference, November 2021.
+        """),
     )
 
     # miscellaneous
@@ -78,19 +84,19 @@ def parse_args() -> argparse.Namespace:
         "-C", "--citation",
         action="version",
         help="show the citation information and exit",
-        version="""
-@inproceedings{heinovski2021scalable,
-    author = {Heinovski, Julian and Buse, Dominik S. and Dressler, Falko},
-    title = {{Scalable Simulation of Platoon Formation Maneuvers with PlaFoSim}},
-    publisher = {IEEE},
-    issn = {2157-9865},
-    isbn = {978-1-66544-450-7},
-    address = {Virtual Conference},
-    booktitle = {13th IEEE Vehicular Networking Conference (VNC 2021), Poster Session},
-    month = {11},
-    year = {2021},
-}
-        """,
+        version=textwrap.dedent("""
+            @inproceedings{heinovski2021scalable,
+                author = {Heinovski, Julian and Buse, Dominik S. and Dressler, Falko},
+                title = {{Scalable Simulation of Platoon Formation Maneuvers with PlaFoSim}},
+                publisher = {IEEE},
+                issn = {2157-9865},
+                isbn = {978-1-66544-450-7},
+                address = {Virtual Conference},
+                booktitle = {13th IEEE Vehicular Networking Conference (VNC 2021), Poster Session},
+                month = {11},
+                year = {2021},
+            }
+        """),
     )
     parser.add_argument(
         "-V", "--version",
@@ -125,9 +131,9 @@ def parse_args() -> argparse.Namespace:
     parser.add_argument(
         '--log-level',
         type=str,
-        default="warn",
-        choices=["warn", "info", "debug"],
-        help="Whether to enable debug output"
+        default=logging.getLevelName(DEFAULTS['log_level']).lower(),
+        choices=["error", "warn", "info", "debug", "trace"],
+        help="The minimum level of logs to be printed",
     )
     parser.add_argument(
         '--start',
@@ -149,35 +155,38 @@ def main():
     args = parse_args()
 
     # TODO add custom filter that prepends the log entry with the step time
-    logging.basicConfig(level=getattr(LOG, args.log_level.upper(), None), format="%(levelname)s: %(message)s")
+    logging.basicConfig(level=args.log_level.upper(), format="%(levelname)s [%(name)s]: %(message)s")
 
     start_gui(config=args.sumo_config)
 
-    LOG.info("Replaying vehicle trace")
+    LOG.info("Replaying vehicle trace...")
 
     traces = pandas.read_csv(args.trace_file).astype({'step': int})
     assert not traces.empty
 
     min_step = max(traces.step.min(), args.start)
     max_step = min(traces.step.max(), args.end) if args.end != -1 else traces.step.max()
+    LOG.debug(f"Running from {min_step}s to {max_step}s...")
 
     if min_step > 0:
-        gui_step(min_step - 1)  # TODO consider various step lengths
+        gui_step(min_step)  # TODO consider various step lengths
 
     import traci
     for step in tqdm(range(min_step, max_step), desc="Trace progress", unit='step'):
         # simulate vehicles from trace file
         for vehicle in traces.loc[traces.step == step].itertuples():
+            color = hex2rgb(vehicle.color) if 'color' in traces else (0, 255, 0)  # allow traces without color
             if str(vehicle.id) not in traci.vehicle.getIDList():
                 add_gui_vehicle(
                     vehicle.id,
                     vehicle.position,
                     vehicle.lane,
                     vehicle.speed,
-                    # color=vehicle.color,  # TODO add vehicle color to trace file
+                    color=color,
                     track=vehicle.id == args.track_vehicle,
                 )
             move_gui_vehicle(vehicle.id, vehicle.position, vehicle.lane, vehicle.speed)
+            change_gui_vehicle_color(vehicle.id, color)
 
         # remove vehicles not in trace file (keep vehicles in trace file)
         prune_vehicles(keep_vids=list(traces.loc[traces.step == step]['id']))
