@@ -33,6 +33,7 @@ from .statistics import (
     record_vehicle_platoon_maneuvers,
     record_vehicle_platoon_trace,
 )
+from .util import round_to_next_base
 from .vehicle import Vehicle
 from .vehicle_type import VehicleType
 
@@ -219,6 +220,8 @@ class PlatooningVehicle(Vehicle):
 
         if self._cf_model == CF_Model.ACC:
             return self._acc_headway_time
+        elif self._cf_model == CF_Model.CACC:
+            return -1
         return super().desired_headway_time
 
     @property
@@ -299,10 +302,7 @@ class PlatooningVehicle(Vehicle):
             The maneuver status
         """
 
-        if self._in_maneuver and var:
-            # we can only start a new maneuver if we are not already in one
-            LOG.warning(f"{self.vid} is already in a maneuver")
-            return
+        assert var != self._in_maneuver, f"Maneuver request {var} does not match maneuver status {self._in_maneuver} of {self._vid}!"
         self._in_maneuver = var
 
     @property
@@ -391,8 +391,10 @@ class PlatooningVehicle(Vehicle):
 
         # statistic recording
 
+        # TODO use pre_filled flag
         if not self._simulator._record_prefilled and self._depart_time == -1:
             # we do not record statistics for pre-filled vehicles
+            LOG.debug(f"Not recording statistics for pre-filled vehicle {self._vid}")
             return
 
         candidates_found_avg = (
@@ -563,7 +565,7 @@ class PlatooningVehicle(Vehicle):
 
         assert not self.is_in_platoon()
 
-        LOG.info(f"{self._vid} is trying to join platoon {platoon_id} (leader {leader_id})")
+        LOG.trace(f"{self._vid} is trying to join platoon {platoon_id} (leader {leader_id})")
         self._joins_attempted += 1
 
         assert not self.in_maneuver
@@ -616,7 +618,7 @@ class PlatooningVehicle(Vehicle):
 
         last = leader.platoon.last
         new_position = last.rear_position - self._cacc_spacing
-        LOG.debug(f"{self._vid}'s new position is ({new_position},{leader.platoon.lane}) (current {self._position},{self._lane})")
+        LOG.trace(f"{self._vid}'s new position is ({new_position},{leader.platoon.lane}) (current {self._position},{self._lane})")
 
         if new_position < self.length:
             # we cannot join since we would be outside of the road
@@ -702,8 +704,10 @@ class PlatooningVehicle(Vehicle):
 
         # delay teleport by approach duration
         if self._simulator._delay_teleports and total_approach_time > 0:
+            # round approach time to next simulation step
+            total_approach_time = round_to_next_base(total_approach_time, self._simulator.step_length)
             # the platoon will be driving while we are waiting
-            new_position = new_position + total_approach_time * leader.platoon.speed
+            new_position += total_approach_time * leader.platoon.speed
             if new_position >= self._simulator.road_length:
                 LOG.warning(f"{self._vid}'s new position would be outside of the road! Aborting the join maneuver")
                 self.in_maneuver = False
@@ -714,7 +718,8 @@ class PlatooningVehicle(Vehicle):
                 self._joins_aborted_road_end += 1
                 return
 
-            # check if the new position (+ the distance in one time step) would be outside of the trip
+            # check if the new position would be outside of the trip
+            # NOTE: we need to add the distance the vehicle drives within one step as well in order to avoid that the vehicles finishes within the next step after the maneuver
             if new_position + self._simulator.step_length * self.speed >= self._arrival_position:
                 LOG.warning(f"{self._vid}'s new position would be outside of its trip! Aborting the join maneuver")
                 self.in_maneuver = False
@@ -730,10 +735,10 @@ class PlatooningVehicle(Vehicle):
             self._join_data_last = last
             self._join_data_new_position = new_position
             leader._joiner = self
-            LOG.debug(f"Scheduled the teleport for vehicle {self._vid} to {self._join_approach_step} ({new_position})")
+            LOG.trace(f"Scheduled the teleport of vehicle {self._vid} to step {self._join_approach_step} (new position: {new_position})")
         else:
             # perform the teleport now
-            LOG.debug(f"The teleport for vehicle {self._vid} will be performed instantaneous.")
+            LOG.trace(f"The teleport for vehicle {self._vid} will be performed instantaneous.")
             self._join_teleport(leader, last, new_position)
 
     def _join_teleport(self, leader: 'PlatooningVehicle', last: 'PlatooningVehicle', new_position: float):
@@ -748,7 +753,7 @@ class PlatooningVehicle(Vehicle):
             The new position for joining the target platoon
         """
 
-        LOG.debug(f"Continuing the join maneuver for vehicle {self._vid} -> {leader.platoon.platoon_id} ({leader.vid})")
+        LOG.trace(f"Continuing the join maneuver for vehicle {self._vid} -> {leader.platoon.platoon_id} ({leader.vid})")
 
         if leader.position >= leader.arrival_position:
             # the leader is gone (or will be gone soon)
@@ -769,7 +774,7 @@ class PlatooningVehicle(Vehicle):
 
         # re-calculate new position
         new_position = last.rear_position - self._cacc_spacing
-        LOG.debug(f"{self._vid}'s new position is ({new_position},{leader.platoon.lane}) (current {self._position},{self._lane})")
+        LOG.trace(f"{self._vid}'s new position is ({new_position},{leader.platoon.lane}) (current {self._position},{self._lane})")
 
         assert new_position >= self.length
         assert new_position >= self._depart_position
@@ -934,14 +939,14 @@ class PlatooningVehicle(Vehicle):
             vehicle._platoon = leader.platoon
             vehicle._cf_target_speed = vehicle._platoon.desired_speed
 
-        LOG.info(f"{self._vid} joined platoon {leader.platoon.platoon_id} (leader: {leader.vid})")
+        LOG.debug(f"{self._vid} joined platoon {leader.platoon.platoon_id} (leader: {leader.vid})")
 
         self.in_maneuver = False
         leader.in_maneuver = False
 
     def calculate_approaching_time(self, target_position: float, target_speed: float) -> float:
         """
-        Calculate approximate time to approacht the target position at target speed
+        Calculate approximate time to approach the target position at target speed
 
         Parameters
         ----------
@@ -1009,18 +1014,18 @@ class PlatooningVehicle(Vehicle):
         current_position = self._position
         if current_position != new_position:
             self._position = new_position
-            LOG.debug(f"{self._vid} teleported to {self._position} (from {current_position}, {self._position - current_position}m)")
+            LOG.trace(f"{self._vid} teleported to {self._position} (from {current_position}, {self._position - current_position}m)")
             self._joins_teleport_position += 1
         current_lane = self._lane
         if current_lane != new_lane:
             self._lane = new_lane
-            LOG.debug(f"{self._vid} switched to lane {self._lane} (from {current_lane})")
+            LOG.trace(f"{self._vid} switched to lane {self._lane} (from {current_lane})")
             self._joins_teleport_lane += 1
         current_speed = self._speed
         self._cf_target_speed = new_speed
         if current_speed != new_speed:
             self._speed = new_speed
-            LOG.debug(f"{self._vid} changed speed to {self._speed}m/s (from {current_speed}m/s)")
+            LOG.trace(f"{self._vid} changed speed to {self._speed}m/s (from {current_speed}m/s)")
             self._joins_teleport_speed += 1
 
     def _leave(self):
@@ -1037,7 +1042,7 @@ class PlatooningVehicle(Vehicle):
 
         leader = self._platoon.leader
 
-        LOG.info(f"{self._vid} is trying to leave platoon {self._platoon.platoon_id} (leader {leader.vid})")
+        LOG.trace(f"{self._vid} is trying to leave platoon {self._platoon.platoon_id} (leader {leader.vid})")
         self._leaves_attempted += 1
 
         if leader.in_maneuver:
@@ -1061,9 +1066,8 @@ class PlatooningVehicle(Vehicle):
 
         self.in_maneuver = True
         self._platoon_role = PlatoonRole.LEAVER
-        leader.in_maneuver = True
 
-        if self is self._platoon.leader:
+        if self is leader:
             # leave at front
             self._leaves_front += 1
 
@@ -1071,7 +1075,7 @@ class PlatooningVehicle(Vehicle):
                 # tell the only follower to drive individually
                 follower = self._platoon.last
                 assert not follower.in_maneuver
-                LOG.debug(f"Only {follower.vid} is left in the platoon {self._platoon.platoon_id}. Thus, we are going to destroy the entire platoon.")
+                LOG.trace(f"Only {follower.vid} is left in the platoon {self._platoon.platoon_id}. Thus, we are going to destroy the entire platoon.")
                 follower._platoon_role = PlatoonRole.NONE
                 follower._cf_model = CF_Model.ACC
                 follower._cf_target_speed = follower._desired_speed
@@ -1097,11 +1101,12 @@ class PlatooningVehicle(Vehicle):
                 LOG.debug(f"{new_leader.vid} became leader of platoon {new_leader.platoon.platoon_id}")
         elif self is self._platoon.last:
             # leave at back
+            leader.in_maneuver = True
             self._leaves_back += 1
 
             if self._platoon.size == 2:
                 # tell the current leader to drive individually
-                LOG.debug(f"Only the current leader {leader.vid} is left in the platoon {self._platoon.platoon_id}. Thus, we are going to destroy the entire platoon.")
+                LOG.trace(f"Only the current leader {leader.vid} is left in the platoon {self._platoon.platoon_id}. Thus, we are going to destroy the entire platoon.")
                 leader._platoon_role = PlatoonRole.NONE
                 leader._cf_model = CF_Model.ACC  # TODO superfluous?
                 leader._cf_target_speed = leader._desired_speed  # TODO superfluous?
@@ -1121,6 +1126,7 @@ class PlatooningVehicle(Vehicle):
                 leader._leaves_front += 1
         else:
             # leave in the middle
+            leader.in_maneuver = True
             self._leaves_arbitrary += 1
 
             # we do not need to switch lanes if we arrived
@@ -1170,7 +1176,7 @@ class PlatooningVehicle(Vehicle):
                 vehicle._platoon = self._platoon
 
         # leave
-        LOG.info(f"{self._vid} left platoon {self._platoon.platoon_id} (leader {self._platoon.leader.vid})")
+        LOG.debug(f"{self._vid} left platoon {self._platoon.platoon_id} (leader {self._platoon.leader.vid})")
         self._platoon_role = PlatoonRole.NONE  # the current platoon role
         self._platoon = Platoon(self._vid, [self], self._desired_speed)  # use explicit individual desired speed
         self._cf_model = CF_Model.ACC  # not necessary, but we still do it explicitly
@@ -1180,7 +1186,8 @@ class PlatooningVehicle(Vehicle):
             change_gui_vehicle_color(self._vid, self._color)
 
         self.in_maneuver = False
-        leader.in_maneuver = False
+        if self is not leader:
+            leader.in_maneuver = False
 
         assert self._last_platoon_join_time >= 0
         self._time_in_platoon += self._simulator.step - self._last_platoon_join_time
@@ -1263,4 +1270,4 @@ class PlatooningVehicle(Vehicle):
         """
 
         # TODO add contents to the neighbor table
-        LOG.info(f"{self._vid} received an advertisement from {advertisement.origin}")
+        LOG.debug(f"{self._vid} received an advertisement from {advertisement.origin}")
