@@ -467,7 +467,7 @@ class Simulator:
         self._collisions = collisions  # whether to check for collisions
         if random_seed < 0:
             random_seed = random.randint(0, 10000)
-        LOG.info(f"Using random seed {random_seed}")
+        LOG.debug(f"Using random seed {random_seed}")
         self._rng = random.Random(random_seed)
         self._progress = progress  # whether to enable the (simulation) progress bar
 
@@ -526,7 +526,7 @@ class Simulator:
         # TODO log generation parameters
         if pre_fill:
             self._generate_vehicles()
-            self._last_vehicle_id = list(self._vehicles.keys())[-1] + 1
+            self._last_vehicle_id = list(self._vehicles.keys())[-1]
             self._number_of_prefilled_vehicles = len(self._vehicles)
         else:
             self._number_of_prefilled_vehicles = 0
@@ -725,7 +725,7 @@ class Simulator:
     def _generate_vehicles(self):
         """Adds pre-filled vehicles to the simulation."""
 
-        LOG.info(f"Pre-filling the road network with {self._number_of_vehicles} vehicles")
+        LOG.debug(f"Pre-filling the road network with {self._number_of_vehicles} vehicles")
 
         assert not self._vehicles
         assert self._last_vehicle_id == -1
@@ -775,11 +775,12 @@ class Simulator:
                     # we do not consider depart interval here since this is supposed to be a snapshot from an earlier point of simulation
                     # make sure to also include the end of the road itself
                     # consider length, equal to departPos="base" in SUMO
-                    depart_position = self._rng.uniform(vtype._length, self._road_length)
+                    # we assume that a vehicle has to drive at least 1m
+                    depart_position = self._rng.uniform(vtype._length, self._road_length - 1)
                     # always use random lane for pre-filled vehicle
                     depart_lane = self._rng.randrange(0, self._number_of_lanes, 1)
 
-                    LOG.debug(f"Generated random depart position ({depart_position},{depart_lane}) for vehicle {vid}")
+                    LOG.trace(f"Generated random depart position ({depart_position},{depart_lane}) for vehicle {vid}")
 
                     if not self._vehicles:
                         continue
@@ -833,7 +834,7 @@ class Simulator:
                 pre_filled=True,
             )
 
-            LOG.debug(f"Generated vehicle {vid} at {depart_position}-{depart_position - vtype._length},{depart_lane} with {depart_speed}")
+            LOG.trace(f"Generated vehicle {vid} at {depart_position}-{depart_position - vtype._length},{depart_lane} with {depart_speed}")
 
         if self._start_as_platoon:
             self._initialize_prefilled_platoon()
@@ -920,9 +921,9 @@ class Simulator:
 
         # 4) spawn
         LOG.trace(f"Trying to spawn {len(self._vehicle_spawn_queue)} new vehicles")
-        # TODO sort by waiting time
+        # sort by waiting/schedule time (and vid) for fairness
         spawned_vehicles_df, not_spawned_vehicles = compute_vehicle_spawns(
-            vehicles=self._vehicle_spawn_queue,
+            vehicles=sorted(self._vehicle_spawn_queue, key=lambda d: (d['schedule_time'], d['vid'])),
             vdf=vdf,
             ramp_positions=self._ramp_positions,
             current_step=self._step,
@@ -974,7 +975,7 @@ class Simulator:
 
         # 5) enque remaining
         if not_spawned_vehicles:
-            LOG.warning(f"Could not spawn {len(not_spawned_vehicles)} vehicles, putting them back into the queue!")
+            LOG.warning(f"Could not spawn a total of {len(not_spawned_vehicles)} vehicles within this step, putting them back into the queue!")
         self._vehicle_spawn_queue = not_spawned_vehicles
         assert len({v['vid'] for v in self._vehicle_spawn_queue}) == len(self._vehicle_spawn_queue)
 
@@ -1118,7 +1119,7 @@ class Simulator:
             )
             self._infrastructures[iid] = infrastructure
 
-            LOG.info(f"Generated infrastructure {infrastructure} at {position}")
+            LOG.info(f"Generated infrastructure {infrastructure.iid} at {position}")
 
     def _initialize_result_recording(self):
         """Creates output files for all (enabled) statistics and writes the headers."""
@@ -1303,6 +1304,9 @@ class Simulator:
             # spawn vehicle based on given parameters
             self._spawn_vehicles(self._get_vehicles_df())
 
+            vehicles_in_simulator = len(self._vehicles)
+            vehicles_in_queue = len(self._vehicle_spawn_queue)
+
             if self._vehicles:
                 # update the GUI
                 if self._gui and self._step >= self._gui_start:
@@ -1392,7 +1396,7 @@ class Simulator:
 
             # record some periodic statistics
             run_time = end_time - start_time
-            self._statistics(run_time)
+            self._statistics(vehicles_in_simulator, vehicles_in_queue, run_time)
 
             # a new step begins
             self._step += self._step_length
@@ -1407,17 +1411,23 @@ class Simulator:
 
         return self._step
 
-    def _statistics(self, runtime: float):
+    def _statistics(self, vehicles_in_simulator, vehicles_in_queue, runtime: float):
         """Record some period statistics."""
 
         self._avg_number_vehicles = int(
-            (self._values_in_avg_number_vehicles * self._avg_number_vehicles + len(self._vehicles)) /
+            (self._values_in_avg_number_vehicles * self._avg_number_vehicles + vehicles_in_simulator) /
             (self._values_in_avg_number_vehicles + 1)
         )
 
         if self._record_simulation_trace:
             # write continuous simulation traces
-            record_simulation_trace(basename=self._result_base_filename, simulator=self, runtime=runtime)
+            record_simulation_trace(
+                basename=self._result_base_filename,
+                step=self._step,
+                vehicles_in_simulator=vehicles_in_simulator,
+                vehicles_in_queue=vehicles_in_queue,
+                runtime=runtime,
+            )
 
     def _record_lane_changes(self, vdf: pd.DataFrame):
         for row in vdf.query('lane != old_lane').itertuples():
@@ -1509,8 +1519,8 @@ class Simulator:
         return (
             pd.DataFrame([
                 dict(
-                    **{key: vehicle.__dict__[f"_{key}"] for key in fields},
-                    **{key: vehicle._vehicle_type.__dict__[f"_{key}"] for key in vtype_fields},
+                    **{key: vars(vehicle)[f"_{key}"] for key in fields},
+                    **{key: vars(vehicle._vehicle_type)[f"_{key}"] for key in vtype_fields},
                     **get_platoon_data(vehicle),
                     desired_headway_time=vehicle.desired_headway_time,
                     acc_lambda=getattr(vehicle, "_acc_lambda", np.nan),
@@ -1574,12 +1584,12 @@ class Simulator:
     def __str__(self) -> str:
         """Returns a str representation of a simulator instance."""
 
-        sim_dict = self.__dict__.copy()
+        sim_dict = vars(self).copy()
         sim_dict.pop('_vehicles')
         sim_dict.pop('_infrastructures')
         sim_dict.update({'current_number_of_vehicles': len(self._vehicles)})
         sim_dict.update({'current_number_of_infrastructures': len(self._infrastructures)})
-        return str(sim_dict)
+        return str(dict(sorted(sim_dict.items())))
 
     def _finish(self):
         """Cleans up the simulation."""
@@ -1671,6 +1681,8 @@ def compute_vehicle_spawns(
         max_remanining_trip_length = ramp_positions[-1] - position
 
         vehicle_to_spawn_now = None
+        # TODO use vectorized approach
+        # assume sorted by waiting time for fairness
         for v in vehicles:
             if (
                 max_remanining_trip_length >= v['min_trip_length']
@@ -1682,6 +1694,7 @@ def compute_vehicle_spawns(
 
         # vehicles are sorted, so we always pick the longest waiting vehicle first :-)
         if not vehicle_to_spawn_now:
+            # no vehicle from the queue fits to the current spawn position
             continue
 
         vehicles_to_spawn[vehicle_to_spawn_now['vid']] = {
